import dataclasses
from typing import List

from jig.collector.domain.ast import ImportFrom, Import
from jig.collector.domain.values.import_path import ImportPath
from jig.collector.domain.source_file.source_file_path import SourceFilePath


@dataclasses.dataclass(frozen=True)
class ImportPathCollection:
    _paths: List[ImportPath] = dataclasses.field(default_factory=list)

    def __len__(self) -> int:
        return len(self._paths)

    def __contains__(self, item: ImportPath):
        return item in self._paths

    def __add__(self, other: "ImportPathCollection") -> "ImportPathCollection":
        return ImportPathCollection(self._paths + other._paths)

    def __iter__(self):
        return self._paths.__iter__()

    @classmethod
    def build_by_import_ast(cls, import_ast: Import) -> "ImportPathCollection":
        imports = [ImportPath.from_str(name.name) for name in import_ast.names]

        return ImportPathCollection(imports)

    @classmethod
    def build_by_import_from_ast(
        cls, file_path: SourceFilePath, import_from: ImportFrom
    ) -> "ImportPathCollection":

        imports = file_path.import_from_to_import_paths(import_from)
        return cls(_paths=imports)



import dataclasses

from jig.collector.domain.values.module_path import ModulePath


@dataclasses.dataclass(frozen=True)
class ModuleDependency:
    src: ModulePath
    dest: ModulePath

    @classmethod
    def build(cls, src: ModulePath, dest: ModulePath) -> "ModuleDependency":
        return cls(src, dest)

    @classmethod
    def from_str(cls, src: str, dest: str) -> "ModuleDependency":
        return cls(src=ModulePath.from_str(src), dest=ModulePath.from_str(dest))



import setuptools


name = "jig-py"
version = "0.0.12"
description = "Jig for Python"
dependencies = [
    "fire",
    "typed_ast",
    "graphviz",
]

with open("README.md", "r") as fh:
    long_description = fh.read()

packages = [
    package for package in setuptools.find_packages() if package.startswith("jig")
]

namespaces = ["jig"]

setuptools.setup(
    name=name,
    version=version,
    author="Jig-Py Project Team",
    author_email="dev+jig-py-contact@levii.co.jp",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/levii/jig-py",
    packages=packages,
    namespaces=namespaces,
    entry_points={"console_scripts": ["jig-py = jig.cli.main:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=dependencies,
)
